<?php
/*
 * Plugin Name: Fatora Payment Gateway
 * Plugin URI: https://fatora.io/api/#plugin
 * Description: Fatora gateway provides robust features for processing online payments, it processes payments securely and conveniently in real time.
 * Author: Maktapp
 * Author URI: https://maktapp.com
 * Version: 2.2
 * Text Domain: fatora-payment-gateway
 */

/*
 * This action hook registers our PHP class as a WooCommerce fatora payment gateway
 */
add_filter('woocommerce_payment_gateways', 'fatora_add_gateway_class');
function fatora_add_gateway_class($gateways)
{
	$gateways[] = 'Fatora_Payment_Gateway'; // your class name is here
	return $gateways;
}
/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action('plugins_loaded', 'fatora_init_gateway_class');
function fatora_init_gateway_class()
{
	/**
	 * Don't load extension if WooCommerce is not active
	 */
	if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
		return;
	}

	// add  Settings linke
	function Fatora_Payment_Gateway_settings_link($links)
	{

		$links[] = '<a href="' .
			admin_url('admin.php?page=wc-settings&tab=checkout&section=fatora_payment_gateway') .
			'">' . __('Settings') . '</a>';
		return $links;
	}
	add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'Fatora_Payment_Gateway_settings_link');


	class Fatora_Payment_Gateway extends WC_Payment_Gateway
	{

		public $save_token;
		public $api_key;
		public $language;
		public $currency;
		public $success_url;
		public $failure_url;
		public $callback;
		/**
		 * Class constructor
		 */
		
		public function __construct()
		{

			$this->id = 'wcfatora'; // payment gateway plugin ID
			$this->icon = plugins_url('assets/images/fatora.png', __FILE__); // URL of the icon that will be displayed on checkout page near your gateway name
			$this->has_fields = false;
			$this->method_title = __('Fatora Payment Gateway', 'fatora-payment-gateway');
			$this->method_description = __('Fatora gateway provides robust features for processing online payments, it processes payments securely and conveniently in real time.', 'fatora-payment-gateway');

			// gateways can support standard checkout
			$this->supports = array(
				'products'
			);

			// Method with all the options fields
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();
			$this->title = $this->get_option('title');
			$this->description = $this->get_option('description');
			$this->enabled  = $this->get_option('enabled');
			$this->save_token = $this->get_option('save_token');
			$this->api_key  = $this->get_option('api_key');
			$this->language  = $this->get_option('language');
			$this->currency  = $this->get_option('currency');

			$this->success_url  =  $this->get_option('success_url');
			$this->failure_url  =  $this->get_option('failure_url');

			$this->callback = str_replace('https:', 'http:', home_url('/wc-api/fatora_payment_gateway'));

			// This action hook saves the settings
			// allows admin settings to be saved in db, do not remove
			if (version_compare(WC()->version, '2.0.0', '>=')) {
				add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
			} else {
				add_action('woocommerce_update_options_payment_gateways', array($this, 'process_admin_options'));
			}

			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

			// register receipt page
			add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
		}
		/**
		 * Plugin options,
		 */
		public function init_form_fields()
		{

			$this->form_fields = array(
				'enabled' => array(
					'title'       => __('Enable/Disable', 'fatora-payment-gateway'),
					'label'       => __('Enable Fatora Gateway', 'fatora-payment-gateway'),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => __('Title'),
					'type'        => 'text',
					'description' => __('This title appears on the checkout page.', 'fatora-payment-gateway'),
					'default'     => __('Fatora Payment Gateway', 'fatora-payment-gateway'),
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => __('Description'),
					'type'        => 'textarea',
					'description' => __('This title appears on the checkout page.', 'fatora-payment-gateway'),
					'default'     => __('Pay with your credit card via Fatora  our super-cool payment gateway.', 'fatora-payment-gateway'),
				),

				'api_key' => array(
					'title'       => __('Live API Key', 'fatora-payment-gateway'),
					'type'        => 'text'
				),
				'success_url' => array(
					'title'       => __('Success Page URL', 'fatora-payment-gateway'),
					'type'        => 'text'
				),
				'failure_url' => array(
					'title'       => __('Failure Page URL', 'fatora-payment-gateway'),
					'type'        => 'text'
				),
				'language' => array(
					'title'       => __('Language'),
					'label'       => __('Language'),
					'type'        => 'text',
					'description' => __('It is determine if the payment gateway will display in Arabic or in English', 'fatora-payment-gateway'),
					'default' => 'ar'
				),
				'currency' => array(
					'title'       => __('Currency', 'fatora-payment-gateway'),
					'label'       => __('Currency', 'fatora-payment-gateway'),
					'type'        => 'text',
					'description' => __('Currency used for the order. Use the three-character ISO Standard Currency Codes e.g. USD, default is QAR', 'fatora-payment-gateway'),
					'default' => 'QAR'
				)
			);
		}

		/**
		 * You will need it if you want your custom credit card form,
		 */
		public function payment_fields()
		{
		}


		public function receipt_page($order_id)
		{
			$order = wc_get_order($order_id);

			if (!$order) {
				// Handle invalid order ID
				return;
			}

			$total = $order->get_total();
			$currency = $this->currency;

			$client_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
			$client_phone = $order->get_billing_phone();
			$client_email = $order->get_billing_email();

			$language = $this->language === "en_US" || $this->language === "en" ? 'en' : 'ar';

			$validationCode = $order_id . '-' . uniqid();
			update_post_meta($order_id, 'validation-code-text', $validationCode);

			$args = array(
				'headers'  => array(
					'Content-Type' => 'application/json',
					'api_key' => $this->api_key
				),
				'body'          => json_encode(array(
					'cmd' => '_notify-synch',
					'amount'  => $total,
					'currency' => $currency,
					'order_id' => $order_id,
					'client'  => array(
						'name' => $client_name,
						'phone' => $client_phone,
						'email' => $client_email
					),
					"language" => $language,
					"success_url" => $this->success_url,
					"failure_url" => $this->failure_url,
					"save_token" => false,
					"note" => "some additional info",
				)),
				'timeout'       => 60,
				'httpversion'   => '1.1',
				'user-agent'    => '1.0',
			);

			$url = 'https://api.fatora.io/v1/payments/checkout';
			$response = wp_remote_post($url, $args);

			if (is_wp_error($response)) {
				// Handle API request error
				$error_message = $response->get_error_message();

				// Log the error message
				error_log('Fatora API Request Error: ' . $error_message);

				// Optionally, you can also display a user-friendly error message on the front end
				wc_add_notice(__('There was an error processing your request. Please try again later or contact support.', 'fatora-payment-gateway'), 'error');

				// Redirect the user to an error page or back to the checkout page
				wp_safe_redirect(wc_get_checkout_url());
				exit;
			}

			$json_data = json_decode($response['body']);

			if ($json_data->status == "SUCCESS") {
				$url = $json_data->result->checkout_url;
				echo '
					<script type="text/javascript">
						window.location.href="' . esc_js($url) . '";
					</script>
					<noscript>
						<meta http-equiv="refresh" content="0;url=' . esc_attr($url) . '">
						<a href="' . esc_attr($url) . '">Click here to proceed</a>
					</noscript>';
			} else {
				$error_code = ""; // Sanitize and validate as necessary
				$description = sprintf("?error_code=%s&description=%s", $error_code, $json_data->error->description);
				$failure_url = $this->failure_url . $description;
				echo '
					<script type="text/javascript">
						window.location.href="' . esc_js($failure_url) . '";
					</script>
					<noscript>
						<meta http-equiv="refresh" content="0;url=' . esc_attr($failure_url) . '">
						<a href="' . esc_attr($failure_url) . '">Click here to proceed</a>
					</noscript>';
			}
		}

		/*
		 * We're processing the payments here, everything about it is in Step 5
		 */
		public function process_payment($order_id)
		{
			$order = wc_get_order($order_id);

			if (!$order || is_wp_error($order)) {
				wc_add_notice(__('Payment error:', 'text-domain') . ' Order not found.', 'error');
				return array('result' => 'fail', 'redirect' => '');
			}

			$redirect_url = add_query_arg(array(
				'order' => $order_id,
				'key' => $order->get_order_key(),
			), wc_get_checkout_url() . 'order-received/');

			$result = array(
				'result' => 'success',
				'redirect' => $redirect_url,
			);
			return $result;
		}

		/*
		 * In case you need a webhook, like PayPal IPN etc
		 */
		public function gateway_response()
		{
			if (isset($_REQUEST['order_id']) && isset($_REQUEST['transaction_id'])) {
				$order_id = sanitize_text_field($_REQUEST['order_id']);
				$transaction_id = sanitize_text_field($_REQUEST['transaction_id']);

				// Validate order ID and transaction ID
				if (!is_numeric($order_id) || !ctype_alnum($transaction_id)) {
					// If either the order ID or transaction ID is invalid, log an error and return
					error_log('Invalid order ID or transaction ID received: Order ID=' . $order_id . ', Transaction ID=' . $transaction_id);
					return;
				}

				$response = $this->verify_transaction($order_id, $transaction_id);

				if ($response && isset($response['status']) && $response['status'] === "SUCCESS") {
					$this->handle_successful_payment($order_id, $transaction_id);
				} else {
					$this->handle_failed_payment($response, $order_id, $transaction_id);
				}
			}
		}

		private function verify_transaction($order_id, $transaction_id)
		{
			$args = array(
				'headers' => array(
					'Content-Type' => 'application/json',
					'api_key' => $this->api_key
				),
				'body' => json_encode(array(
					'cmd' => '_notify-synch',
					'order_id' => $order_id,
					'transaction_id' => $transaction_id
				)),
				'timeout' => 60,
				'httpversion' => '1.1',
				'user-agent' => '1.0',
			);

			$url = 'https://api.fatora.io/v1/payments/verify';
			$response = wp_remote_post($url, $args);

			if (is_wp_error($response)) {
				// Handle API request error
				return null;
			}

			return json_decode(wp_remote_retrieve_body($response), true);
		}

		private function handle_successful_payment($order_id, $transaction_id)
		{
			$order = wc_get_order($order_id);

			if ($order) {
				update_post_meta($order_id, 'transaction_id', $transaction_id);
				$order->payment_complete();
				$order->update_status('completed', 'Order payment completed.');
				WC()->cart->empty_cart();
			}
		}

		private function handle_failed_payment($response, $order_id, $transaction_id)
		{
			update_post_meta($order_id, 'transaction_id', $transaction_id);
			$json_data = json_decode($response['body']);
			$error_code = ""; //$json_data->error->error_code ;
			$description = sprintf("?error_code=%s&description=%s", $error_code, $json_data->error->description);
			$failure_url = $this->failure_url . $description;
			wp_safe_redirect($failure_url);
			exit;
		}
	}
}

add_action('template_redirect', 'success_page', 1);
function success_page()
{
	$fatora = new Fatora_Payment_Gateway();
	$success_url  = $fatora->get_option('success_url');
	if (str_contains(get_the_permalink(), $success_url)) {
		if (isset($_REQUEST['order_id'])) {
			$fatora->gateway_response();
		}
	}
}

add_action('init', 'wpdocs_load_textdomain');
/**
 * Load plugin textdomain.
 */
function wpdocs_load_textdomain()
{
	load_plugin_textdomain('fatora-payment-gateway', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
